@@ -1,6 +1,9 @@
 /****
 DIAMOND protein aligner
-Copyright (C) 2013-2017 Benjamin Buchfink <buchfink@gmail.com>
+Copyright (C) 2016-2021 Max Planck Society for the Advancement of Science e.V.
+                        Benjamin Buchfink
+						
+Code developed by Benjamin Buchfink <benjamin.buchfink@tue.mpg.de>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,11 +19,8 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
-#ifndef DIAGONAL_SEGMENT_H_
-#define DIAGONAL_SEGMENT_H_
-
+#pragma once
 #include "translated_position.h"
-#include "../stats/score_matrix.h"
 #include "sequence.h"
 
 struct Diagonal_segment
@@ -225,5 +225,3 @@ struct DiagonalSegment
 	TranslatedPosition i;
 	int j, len, score;
 };
-
-#endif
\ No newline at end of file
