@@ -1,6 +1,6 @@
 /****
 DIAMOND protein aligner
-Copyright (C) 2013-2020 Max Planck Society for the Advancement of Science e.V.
+Copyright (C) 2013-2021 Max Planck Society for the Advancement of Science e.V.
                         Benjamin Buchfink
                         Eberhard Karls Universitaet Tuebingen
 						
@@ -25,25 +25,25 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <vector>
 #include <stddef.h>
 #include "../basic/sequence.h"
+#include "../util/algo/binary_search.h"
 
-template<typename _t, char _pchar = '\xff', size_t _padding = 1lu>
-struct String_set
+template<typename T, char _pchar, size_t _padding = 1lu>
+struct StringSetBase
 {
 
 	enum { PERIMETER_PADDING = 256 };
 	static const char DELIMITER = _pchar;
 
-	String_set():
-		data_ (PERIMETER_PADDING)
-	{ limits_.push_back(PERIMETER_PADDING); }
+	StringSetBase():
+		data_ (PERIMETER_PADDING, _pchar)
+	{
+		limits_.push_back(PERIMETER_PADDING);
+	}
 
 	void finish_reserve()
 	{
 		data_.resize(raw_len() + PERIMETER_PADDING);
-		for(unsigned i=0;i<PERIMETER_PADDING;++i) {
-			data_[i] = _pchar;
-			data_[raw_len()+i] = _pchar;
-		}
+		std::fill(data_.begin() + raw_len(), data_.end(), _pchar);
 	}
 
 	void reserve(size_t n)
@@ -51,6 +51,21 @@ struct String_set
 		limits_.push_back(raw_len() + n + _padding);
 	}
 
+	void reserve(size_t entries, size_t length) {
+		limits_.reserve(entries + 1);
+		data_.reserve(length + 2 * PERIMETER_PADDING + entries * _padding);
+	}
+
+	void clear() {
+		limits_.resize(1);
+		data_.resize(PERIMETER_PADDING);
+	}
+
+	void shrink_to_fit() {
+		limits_.shrink_to_fit();
+		data_.shrink_to_fit();
+	}
+
 	template<typename _it>
 	void push_back(_it begin, _it end)
 	{
@@ -60,19 +75,28 @@ struct String_set
 		data_.insert(data_.end(), _padding, _pchar);
 	}
 
-	void fill(size_t n, _t v)
+	template<typename It>
+	void assign(const size_t i, const It begin, const It end) {
+		std::copy(begin, end, ptr(i));
+	}
+
+	void fill(size_t n, T v)
 	{
 		limits_.push_back(raw_len() + n + _padding);
 		data_.insert(data_.end(), n, v);
 		data_.insert(data_.end(), _padding, _pchar);
 	}
 
-	_t* ptr(size_t i)
+	T* ptr(size_t i)
 	{ return &data_[limits_[i]]; }
 
-	const _t* ptr(size_t i) const
+	const T* ptr(size_t i) const
 	{ return &data_[limits_[i]]; }
 
+	const T* end(size_t i) const {
+		return &data_[limits_[i + 1] - _padding];
+	}
+
 	size_t check_idx(size_t i) const
 	{
 		if (limits_.size() < i + 2)
@@ -83,22 +107,26 @@ struct String_set
 	size_t length(size_t i) const
 	{ return limits_[i+1] - limits_[i] - _padding; }
 
-	size_t get_length() const
+	size_t size() const
 	{ return limits_.size() - 1; }
 
+	bool empty() const {
+		return limits_.size() <= 1;
+	}
+
 	size_t raw_len() const
 	{ return limits_.back(); }
 
 	size_t letters() const
-	{ return raw_len() - get_length() - PERIMETER_PADDING; }
+	{ return raw_len() - size() - PERIMETER_PADDING; }
 
-	_t* data(uint64_t p = 0)
+	T* data(uint64_t p = 0)
 	{ return &data_[p]; }
 
-	const _t* data(uint64_t p = 0) const
+	const T* data(uint64_t p = 0) const
 	{ return &data_[p]; }
 
-	size_t position(const _t* p) const
+	size_t position(const T* p) const
 	{ return p - data(); }
 
 	size_t position(size_t i, size_t j) const
@@ -110,11 +138,20 @@ struct String_set
 		return std::pair<size_t, size_t>(i, p - limits_[i]);
 	}
 
-	const _t* operator[](size_t i) const
+	template<typename It, typename Out, typename Cmp>
+	void local_position_batch(It begin, It end, Out out, Cmp cmp) const {
+		batch_binary_search(begin, end, limits_.begin(), limits_.end(), out, cmp);
+	}
+
+	const T* operator[](size_t i) const
 	{
 		return ptr(i);
 	}
 
+	const T* back() const {
+		return ptr(limits_.size() - 2);
+	}
+
 	typename std::vector<size_t>::const_iterator limits_begin() const {
 		return limits_.begin();
 	}
@@ -123,9 +160,65 @@ struct String_set
 		return limits_.end();
 	}
 
+	struct ConstIterator {
+
+		ConstIterator(const T* data, const size_t* limits):
+			data_(data),
+			limits_(limits)
+		{}
+
+		using iterator_category = std::random_access_iterator_tag;
+		using difference_type = ptrdiff_t;
+		using value_type = T;
+		using pointer = T*;
+		using reference = T&;
+
+		ptrdiff_t operator-(const ConstIterator& it) const {
+			return limits_ - it.limits_;
+		}
+
+		bool operator==(const ConstIterator& it) const {
+			return limits_ == it.limits_;
+		}
+
+		ConstIterator operator+(ptrdiff_t d) const {
+			return { data_ + *(limits_ + d) - *limits_, limits_ + d };
+		}
+
+		bool operator<(const ConstIterator& it) const {
+			return limits_ < it.limits_;
+		}
+
+		ConstIterator& operator+=(ptrdiff_t d) {
+			data_ += *(limits_ + d) - *limits_;
+			limits_ += d;
+			return *this;
+		}
+
+		std::pair<const T*, int64_t> operator[](const ptrdiff_t i) const {
+			return { data_ + limits_[i] - limits_[0], limits_[i + 1] - limits_[i] - _padding };
+		}
+
+	private:
+
+		const T* data_;
+		const size_t* limits_;
+
+	};
+
+	ConstIterator cbegin() const {
+		return ConstIterator(ptr(0), limits_.data());
+	}
+
+	ConstIterator cend() const {
+		return ConstIterator(nullptr, &limits_[size()]);
+	}
+
 private:
 
-	std::vector<_t> data_;
+	std::vector<T> data_;
 	std::vector<size_t> limits_;
 
 };
+
+using StringSet = StringSetBase<char, '\0'>;
\ No newline at end of file
