@@ -1,6 +1,9 @@
 /****
 DIAMOND protein aligner
-Copyright (C) 2013-2018 Benjamin Buchfink <buchfink@gmail.com>
+Copyright (C) 2016-2021 Max Planck Society for the Advancement of Science e.V.
+                        Benjamin Buchfink
+						
+Code developed by Benjamin Buchfink <benjamin.buchfink@tue.mpg.de>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,17 +19,17 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
-#ifndef TAXON_LIST_H_
-#define TAXON_LIST_H_
-
+#pragma once
 #include "../util/io/output_file.h"
-#include "../util/io/file_backed_buffer.h"
 #include "../util/data_structures/compact_array.h"
+#include "../util/table.h"
 
-struct TaxonList : public CompactArray<vector<unsigned> >
+template<typename Type, typename Cmp>
+struct ExternalSorter;
+
+struct TaxonList : public CompactArray<vector<unsigned>>
 {
+	typedef std::pair<std::string, uint32_t> T;
 	TaxonList(Deserializer &in, size_t size, size_t data_size);
-	static void build(OutputFile &db, FileBackedBuffer &accessions, size_t seqs);
-};
-
-#endif
\ No newline at end of file
+	static void build(OutputFile &db, ExternalSorter<T, std::less<T>>& accessions, size_t seqs, Table& stats);
+};
\ No newline at end of file
